-- CCBreakWarner 1.6, by Wikwocket (wikwocket@gmail.com)
-- 
-- Alerts you when a CC spell you can cast fades or is broken.

local VERSION = "1.6";

-- Print: helper function to send message to default chat frame
local function Print(text)
	if (DEFAULT_CHAT_FRAME) then
		DEFAULT_CHAT_FRAME:AddMessage(text);
	end
end

-- pre-WotLK variables:
local className = nil;
local ccName1 = "(unused)";
local ccName2 = "(unused)";
local ccName3 = "(unused)";

-- OnLoad: register events, slash commands, and hooks, print welcome message
function CCBreakWarnerOnLoad()
	_, className = UnitClass("player");
	if (className == "DRUID") then
		CCBreakWarnerFrame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
		Print("CCBreakWarner " .. VERSION .. " enabled, for Druid CC types");
		ccName1 = CCBW_LOCALE.cyclone;
		ccName2 = CCBW_LOCALE.entanglingRoots;
		ccName3 = CCBW_LOCALE.hibernate;
	elseif (className == "HUNTER") then
		CCBreakWarnerFrame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
		Print("CCBreakWarner " .. VERSION .. " enabled, for Hunter CC types");
		ccName1 = CCBW_LOCALE.freezingTrapEffect;
		ccName2 = CCBW_LOCALE.scareBeast;
		ccName3 = CCBW_LOCALE.wyvernSting;
	elseif (className == "MAGE") then
		CCBreakWarnerFrame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
		Print("CCBreakWarner " .. VERSION .. " enabled, for Mage CC types");
		ccName1 = CCBW_LOCALE.polymorph;
	elseif (className == "PALADIN") then
		CCBreakWarnerFrame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
		Print("CCBreakWarner " .. VERSION .. " enabled, for Paladin CC types");
		ccName1 = CCBW_LOCALE.repentance;
		ccName2 = CCBW_LOCALE.turnEvil;
		ccName3 = CCBW_LOCALE.turnUndead;
	elseif (className == "PRIEST") then
		CCBreakWarnerFrame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
		Print("CCBreakWarner " .. VERSION .. " enabled, for Priest CC types");
		ccName1 = CCBW_LOCALE.shackleUndead;
	elseif (className == "ROGUE") then
		CCBreakWarnerFrame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
		Print("CCBreakWarner " .. VERSION .. " enabled, for Rogue CC types");
		ccName1 = CCBW_LOCALE.blind;
		ccName2 = CCBW_LOCALE.sap;
	elseif (className == "SHAMAN") then
		CCBreakWarnerFrame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
		Print("CCBreakWarner " .. VERSION .. " enabled, for Shaman CC types");
		ccName1 = CCBW_LOCALE.hex;
	elseif (className == "WARLOCK") then
		CCBreakWarnerFrame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
		Print("CCBreakWarner " .. VERSION .. " enabled, for Warlock CC types");
		ccName1 = CCBW_LOCALE.banish;
		ccName2 = CCBW_LOCALE.fear;
		--ccName3 = CCBW_LOCALE.seduction; (actually check this seperately)
	end
end

-- OnEvent: combat log event
function CCBreakWarnerOnEvent()
	-- combat log event: check for CC debuffs fading
	--   arg list for all combat log events: timestamp, eventType, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags
	--   additional args for SPELL_AURA_REMOVED: spellId, spellName, spellSchool, auraType
	if (event == "COMBAT_LOG_EVENT_UNFILTERED") then
		--Print("CCBW DEBUG: COMBAT_LOG_EVENT_UNFILTERED arg2=" .. arg2);
		if (arg2 == "SPELL_AURA_REMOVED" and arg12 == "DEBUFF") then -- arg2 = eventType, arg12 = auraType
			--Print("CCBW DEBUG: SPELL_AURA_REMOVED arg5=" .. arg5 .. " arg7=" .. arg7 .. " arg12=" .. arg12);
			-- was it my debuff fading?
			-- SPELL_AURA_REMOVED, with arg5=srcFlags = 1297 (511 hex)
			--     (apply 0x0000FFFF mask to arg5 (via modulo) to remove special case flags)
			--     (5 = COMBATLOG_OBJECT_CONTROL_PLAYER & COMBATLOG_OBJECT_TYPE_PLAYER)
			--     (1 = COMBATLOG_OBJECT_REACTION_FRIENDLY)
			--     (1 = COMBATLOG_OBJECT_AFFILIATION_MINE)
			if (arg5 % 0x00010000 == 1297) then
				-- was it a CC debuff? (arg10 is spell name, arg7 is target name)
				if (arg10 == ccName1) then
					CCBreakWarnerNotify(ccName1 .. " has broken on " .. arg7 .. "!")
				elseif (arg10 == ccName2) then
					CCBreakWarnerNotify(ccName2 .. " has broken on " .. arg7 .. "!")
				elseif (arg10 == ccName3) then
					CCBreakWarnerNotify(ccName3 .. " has broken on " .. arg7 .. "!")
				end
			end
			-- was it my pet's debuff fading?
			-- SPELL_AURA_REMOVED, with arg5=srcFlags = 4369 (1111 hex)
			--     (apply 0x0000FFFF mask to arg5 (via modulo) to remove special case flags)
			--     (1 = COMBATLOG_OBJECT_TYPE_PET)
			--     (1 = COMBATLOG_OBJECT_CONTROL_PLAYER)
			--     (1 = COMBATLOG_OBJECT_REACTION_FRIENDLY)
			--     (1 = COMBATLOG_OBJECT_AFFILIATION_MINE)
			if (arg5 % 0x00010000 == 4369) then
				-- was it a succubus's seduction debuff? (arg10 is spell name, arg7 is target name)
				if (arg10 == CCBW_LOCALE.seduction) then
					CCBreakWarnerNotify(CCBW_LOCALE.seduction .. " has broken on " .. arg7 .. "!")
				end
			end
		end
	end
end

-- Notify: alert with text and sound
function CCBreakWarnerNotify(TextMessage)
	Print(TextMessage);
	if (UIErrorsFrame) then
		UIErrorsFrame:AddMessage(TextMessage, 1, 1, 0, 1.0, UIERRORS_HOLD_TIME);
	end
	PlaySoundFile("Sound\\Doodad\\CollectorTubeShutdown.wav");
end
